@@ -12,9 +12,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GDK_DND_PRIVATE_H__
@@ -64,6 +62,7 @@ struct _GdkDragContextClass {
                                 gboolean         success,
                                 guint32          time_);
   gboolean    (*drop_status)   (GdkDragContext  *context);
+  GdkWindow*  (*get_drag_window) (GdkDragContext *context);
 };
 
 struct _GdkDragContext {
@@ -75,6 +74,7 @@ struct _GdkDragContext {
   gboolean is_source;
   GdkWindow *source_window;
   GdkWindow *dest_window;
+  GdkWindow *drag_window;
 
   GList *targets;
   GdkDragAction actions;
