@@ -29,7 +29,7 @@
 #include "gdkkeysyms.h"
 #include "gdkdeviceprivate.h"
 #include "gdkdevicemanagerprivate.h"
-#include "pointer-gestures-client-protocol.h"
+#include "pointer-gestures-unstable-v1-client-protocol.h"
 
 #include <xkbcommon/xkbcommon.h>
 
@@ -59,8 +59,8 @@ struct _GdkWaylandDeviceData
   struct wl_pointer *wl_pointer;
   struct wl_keyboard *wl_keyboard;
   struct wl_touch *wl_touch;
-  struct _wl_pointer_gesture_swipe *wl_pointer_gesture_swipe;
-  struct _wl_pointer_gesture_pinch *wl_pointer_gesture_pinch;
+  struct zwp_pointer_gesture_swipe_v1 *wp_pointer_gesture_swipe;
+  struct zwp_pointer_gesture_pinch_v1 *wp_pointer_gesture_pinch;
 
   GdkDisplay *display;
   GdkDeviceManager *device_manager;
@@ -96,7 +96,6 @@ struct _GdkWaylandDeviceData
   guint32 repeat_count;
   GSettings *keyboard_settings;
 
-  GdkCursor *grab_cursor;
   guint cursor_timeout_id;
   guint cursor_image_index;
   guint cursor_image_delay;
@@ -206,12 +205,7 @@ gdk_wayland_device_update_window_cursor (GdkWaylandDeviceData *wd)
   guint next_image_index, next_image_delay;
   gboolean retval = G_SOURCE_REMOVE;
 
-  if (wd->grab_cursor)
-    {
-      buffer = _gdk_wayland_cursor_get_buffer (wd->grab_cursor, 0,
-                                               &x, &y, &w, &h, &scale);
-    }
-  else if (wd->cursor)
+  if (wd->cursor)
     {
       buffer = _gdk_wayland_cursor_get_buffer (wd->cursor, wd->cursor_image_index,
                                                &x, &y, &w, &h, &scale);
@@ -248,13 +242,6 @@ gdk_wayland_device_update_window_cursor (GdkWaylandDeviceData *wd)
       wl_surface_commit (wd->pointer_surface);
     }
 
-  if (wd->grab_cursor)
-    {
-      /* We admit only static icons during drags so far */
-      gdk_wayland_device_stop_window_cursor_animation (wd);
-      return retval;
-    }
-
   next_image_index =
     _gdk_wayland_cursor_get_next_image_index (wd->cursor,
                                               wd->cursor_image_index,
@@ -545,10 +532,10 @@ gdk_wayland_device_grab (GdkDevice    *device,
                                               wayland_device->wl_seat,
                                               time_);
 
-      g_clear_object (&wayland_device->grab_cursor);
+      g_clear_object (&wayland_device->cursor);
 
       if (cursor)
-        wayland_device->grab_cursor = g_object_ref (cursor);
+        wayland_device->cursor = g_object_ref (cursor);
 
       gdk_wayland_device_update_window_cursor (wayland_device);
     }
@@ -587,7 +574,6 @@ gdk_wayland_device_ungrab (GdkDevice *device,
   else
     {
       /* Device is a pointer */
-      g_clear_object (&wayland_device->grab_cursor);
       gdk_wayland_device_update_window_cursor (wayland_device);
 
       if (wayland_device->pointer_grab_window)
@@ -1822,12 +1808,12 @@ emit_gesture_swipe_event (GdkWaylandDeviceData    *device,
 }
 
 static void
-gesture_swipe_begin (void                             *data,
-                     struct _wl_pointer_gesture_swipe *swipe,
-                     uint32_t                          serial,
-                     uint32_t                          time,
-                     struct wl_surface                *surface,
-                     uint32_t                          fingers)
+gesture_swipe_begin (void                                *data,
+                     struct zwp_pointer_gesture_swipe_v1 *swipe,
+                     uint32_t                             serial,
+                     uint32_t                             time,
+                     struct wl_surface                   *surface,
+                     uint32_t                             fingers)
 {
   GdkWaylandDeviceData *device = data;
   GdkWaylandDisplay *display = GDK_WAYLAND_DISPLAY (device->display);
@@ -1841,11 +1827,11 @@ gesture_swipe_begin (void                             *data,
 }
 
 static void
-gesture_swipe_update (void                             *data,
-                      struct _wl_pointer_gesture_swipe *swipe,
-                      uint32_t                          time,
-                      wl_fixed_t                        dx,
-                      wl_fixed_t                        dy)
+gesture_swipe_update (void                                *data,
+                      struct zwp_pointer_gesture_swipe_v1 *swipe,
+                      uint32_t                             time,
+                      wl_fixed_t                           dx,
+                      wl_fixed_t                           dy)
 {
   GdkWaylandDeviceData *device = data;
 
@@ -1858,11 +1844,11 @@ gesture_swipe_update (void                             *data,
 }
 
 static void
-gesture_swipe_end (void                             *data,
-                   struct _wl_pointer_gesture_swipe *swipe,
-                   uint32_t                          serial,
-                   uint32_t                          time,
-                   int32_t                           cancelled)
+gesture_swipe_end (void                                *data,
+                   struct zwp_pointer_gesture_swipe_v1 *swipe,
+                   uint32_t                             serial,
+                   uint32_t                             time,
+                   int32_t                              cancelled)
 {
   GdkWaylandDeviceData *device = data;
   GdkWaylandDisplay *display = GDK_WAYLAND_DISPLAY (device->display);
@@ -1926,12 +1912,12 @@ emit_gesture_pinch_event (GdkWaylandDeviceData    *device,
 }
 
 static void
-gesture_pinch_begin (void                             *data,
-                     struct _wl_pointer_gesture_pinch *pinch,
-                     uint32_t                          serial,
-                     uint32_t                          time,
-                     struct wl_surface                *surface,
-                     uint32_t                          fingers)
+gesture_pinch_begin (void                                *data,
+                     struct zwp_pointer_gesture_pinch_v1 *pinch,
+                     uint32_t                             serial,
+                     uint32_t                             time,
+                     struct wl_surface                   *surface,
+                     uint32_t                             fingers)
 {
   GdkWaylandDeviceData *device = data;
   GdkWaylandDisplay *display = GDK_WAYLAND_DISPLAY (device->display);
@@ -1944,13 +1930,13 @@ gesture_pinch_begin (void                             *data,
 }
 
 static void
-gesture_pinch_update (void                             *data,
-                      struct _wl_pointer_gesture_pinch *pinch,
-                      uint32_t                          time,
-                      wl_fixed_t                        dx,
-                      wl_fixed_t                        dy,
-                      wl_fixed_t                        scale,
-                      wl_fixed_t                        rotation)
+gesture_pinch_update (void                                *data,
+                      struct zwp_pointer_gesture_pinch_v1 *pinch,
+                      uint32_t                             time,
+                      wl_fixed_t                           dx,
+                      wl_fixed_t                           dy,
+                      wl_fixed_t                           scale,
+                      wl_fixed_t                           rotation)
 {
   GdkWaylandDeviceData *device = data;
 
@@ -1964,11 +1950,11 @@ gesture_pinch_update (void                             *data,
 }
 
 static void
-gesture_pinch_end (void                             *data,
-                   struct _wl_pointer_gesture_pinch *pinch,
-                   uint32_t                          serial,
-                   uint32_t                          time,
-                   int32_t                           cancelled)
+gesture_pinch_end (void                                *data,
+                   struct zwp_pointer_gesture_pinch_v1 *pinch,
+                   uint32_t                             serial,
+                   uint32_t                             time,
+                   int32_t                              cancelled)
 {
   GdkWaylandDeviceData *device = data;
   GdkWaylandDisplay *display = GDK_WAYLAND_DISPLAY (device->display);
@@ -2010,13 +1996,13 @@ static const struct wl_touch_listener touch_listener = {
   touch_handle_cancel
 };
 
-static const struct _wl_pointer_gesture_swipe_listener gesture_swipe_listener = {
+static const struct zwp_pointer_gesture_swipe_v1_listener gesture_swipe_listener = {
   gesture_swipe_begin,
   gesture_swipe_update,
   gesture_swipe_end
 };
 
-static const struct _wl_pointer_gesture_pinch_listener gesture_pinch_listener = {
+static const struct zwp_pointer_gesture_pinch_v1_listener gesture_pinch_listener = {
   gesture_pinch_begin,
   gesture_pinch_update,
   gesture_pinch_end
@@ -2060,21 +2046,21 @@ seat_handle_capabilities (void                    *data,
 
       if (wayland_display->pointer_gestures)
         {
-          device->wl_pointer_gesture_swipe =
-            _wl_pointer_gestures_get_swipe_gesture (wayland_display->pointer_gestures,
-                                                    device->wl_pointer);
-          _wl_pointer_gesture_swipe_set_user_data (device->wl_pointer_gesture_swipe,
-                                                   device);
-          _wl_pointer_gesture_swipe_add_listener (device->wl_pointer_gesture_swipe,
-                                                  &gesture_swipe_listener, device);
+          device->wp_pointer_gesture_swipe =
+            zwp_pointer_gestures_v1_get_swipe_gesture (wayland_display->pointer_gestures,
+                                                       device->wl_pointer);
+          zwp_pointer_gesture_swipe_v1_set_user_data (device->wp_pointer_gesture_swipe,
+                                                      device);
+          zwp_pointer_gesture_swipe_v1_add_listener (device->wp_pointer_gesture_swipe,
+                                                     &gesture_swipe_listener, device);
 
-          device->wl_pointer_gesture_pinch =
-            _wl_pointer_gestures_get_pinch_gesture (wayland_display->pointer_gestures,
-                                                    device->wl_pointer);
-          _wl_pointer_gesture_pinch_set_user_data (device->wl_pointer_gesture_pinch,
-                                                   device);
-          _wl_pointer_gesture_pinch_add_listener (device->wl_pointer_gesture_pinch,
-                                                  &gesture_pinch_listener, device);
+          device->wp_pointer_gesture_pinch =
+            zwp_pointer_gestures_v1_get_pinch_gesture (wayland_display->pointer_gestures,
+                                                       device->wl_pointer);
+          zwp_pointer_gesture_pinch_v1_set_user_data (device->wp_pointer_gesture_pinch,
+                                                      device);
+          zwp_pointer_gesture_pinch_v1_add_listener (device->wp_pointer_gesture_pinch,
+                                                     &gesture_pinch_listener, device);
         }
 
       g_signal_emit_by_name (device_manager, "device-added", device->pointer);
@@ -2276,8 +2262,6 @@ pointer_surface_update_scale (GdkWaylandDeviceData *device)
 
   device->current_output_scale = scale;
 
-  if (device->grab_cursor)
-    _gdk_wayland_cursor_set_scale (device->grab_cursor, scale);
   if (device->cursor)
     _gdk_wayland_cursor_set_scale (device->cursor, scale);
 
