@@ -693,6 +693,24 @@ button_callback (gpointer data)
 
 static UINT taskbar_created_msg = 0;
 static GSList *status_icons = NULL;
+static UINT status_icon_id = 0;
+
+static GtkStatusIcon *
+find_status_icon (UINT id)
+{
+  GSList *rover;
+
+  for (rover = status_icons; rover != NULL; rover = rover->next)
+    {
+      GtkStatusIcon *status_icon = GTK_STATUS_ICON (rover->data);
+      GtkStatusIconPrivate *priv = status_icon->priv;
+
+      if (priv->nid.uID == id)
+        return status_icon;
+    }
+
+  return NULL;
+}
 
 static LRESULT CALLBACK
 wndproc (HWND   hwnd,
@@ -710,7 +728,7 @@ wndproc (HWND   hwnd,
 	  GtkStatusIconPrivate *priv = status_icon->priv;
 
 	  priv->nid.hWnd = hwnd;
-	  priv->nid.uID = GPOINTER_TO_UINT (status_icon);
+	  priv->nid.uID = status_icon_id++;
 	  priv->nid.uCallbackMessage = WM_GTK_TRAY_NOTIFICATION;
 	  priv->nid.uFlags = NIF_MESSAGE;
 
@@ -754,7 +772,7 @@ wndproc (HWND   hwnd,
 	buttondown0:
 	  bc = g_new (ButtonCallbackData, 1);
 	  bc->event = (GdkEventButton *) gdk_event_new (GDK_BUTTON_PRESS);
-	  bc->status_icon = GTK_STATUS_ICON (wparam);
+	  bc->status_icon = find_status_icon (wparam);
 	  build_button_event (bc->status_icon->priv, bc->event, button);
 	  g_idle_add (button_callback, bc);
 	  break;
@@ -780,7 +798,7 @@ wndproc (HWND   hwnd,
 	buttonup0:
 	  bc = g_new (ButtonCallbackData, 1);
 	  bc->event = (GdkEventButton *) gdk_event_new (GDK_BUTTON_RELEASE);
-	  bc->status_icon = GTK_STATUS_ICON (wparam);
+	  bc->status_icon = find_status_icon (wparam);
 	  build_button_event (bc->status_icon->priv, bc->event, button);
 	  g_idle_add (button_callback, bc);
 	  break;
@@ -927,7 +945,7 @@ gtk_status_icon_init (GtkStatusIcon *status_icon)
   memset (&priv->nid, 0, sizeof (priv->nid));
 
   priv->nid.hWnd = create_tray_observer ();
-  priv->nid.uID = GPOINTER_TO_UINT (status_icon);
+  priv->nid.uID = status_icon_id++;
   priv->nid.uCallbackMessage = WM_GTK_TRAY_NOTIFICATION;
   priv->nid.uFlags = NIF_MESSAGE;
 
@@ -1359,6 +1377,7 @@ gtk_status_icon_update_image (GtkStatusIcon *status_icon)
   HICON prev_hicon;
 #endif
   GtkStyleContext *context;
+  cairo_surface_t *surface;
   GtkWidget *widget;
   GdkPixbuf *pixbuf;
   gint round_size;
@@ -1376,7 +1395,9 @@ gtk_status_icon_update_image (GtkStatusIcon *status_icon)
   round_size = round_pixel_size (widget, priv->size);
 
   _gtk_icon_helper_set_pixel_size (priv->icon_helper, round_size);
-  pixbuf = _gtk_icon_helper_ensure_pixbuf (priv->icon_helper, context);
+  surface = gtk_icon_helper_load_surface (priv->icon_helper, context, 1);
+  pixbuf = gdk_pixbuf_get_from_surface (surface, 0, 0, round_size, round_size);
+  cairo_surface_destroy (surface);
 
   if (pixbuf != NULL)
     {
