@@ -97,6 +97,11 @@
  *
  * # CSS nodes
  *
+ * |[<!-- language="plain" -->
+ * label
+ * ╰── accelerator
+ * ]|
+ *
  * Like #GtkLabel, GtkAccelLabel has a main CSS node with the name label.
  * It adds a subnode with name accelerator.
  */
@@ -407,7 +412,7 @@ gtk_accel_label_get_accel_layout (GtkAccelLabel *accel_label)
   if (!attrs)
     attrs = pango_attr_list_new ();
   gtk_style_context_get (context,
-                         gtk_widget_get_state_flags (widget),
+                         gtk_style_context_get_state (context),
                          "font", &font_desc,
                          NULL);
   pango_attr_list_change (attrs, pango_attr_font_desc_new (font_desc));
