@@ -481,9 +481,9 @@ _gtk_css_font_weight_value_try_parse (GtkCssParser *parser)
     }
   /* special cases go here */
   if (_gtk_css_parser_try (parser, "400", TRUE))
-    return _gtk_css_value_ref (&font_weight_values[5]);
+    return _gtk_css_value_ref (&font_weight_values[3]);
   if (_gtk_css_parser_try (parser, "700", TRUE))
-    return _gtk_css_value_ref (&font_weight_values[8]);
+    return _gtk_css_value_ref (&font_weight_values[6]);
 
   return NULL;
 }
@@ -869,9 +869,9 @@ _gtk_css_fill_mode_value_get (const GtkCssValue *value)
   return value->value;
 }
 
-/* GtkCssImageEffect */
+/* GtkCssIconEffect */
 
-static const GtkCssValueClass GTK_CSS_VALUE_IMAGE_EFFECT = {
+static const GtkCssValueClass GTK_CSS_VALUE_ICON_EFFECT = {
   gtk_css_value_enum_free,
   gtk_css_value_enum_compute,
   gtk_css_value_enum_equal,
@@ -880,13 +880,13 @@ static const GtkCssValueClass GTK_CSS_VALUE_IMAGE_EFFECT = {
 };
 
 static GtkCssValue image_effect_values[] = {
-  { &GTK_CSS_VALUE_IMAGE_EFFECT, 1, GTK_CSS_IMAGE_EFFECT_NONE, "none" },
-  { &GTK_CSS_VALUE_IMAGE_EFFECT, 1, GTK_CSS_IMAGE_EFFECT_HIGHLIGHT, "highlight" },
-  { &GTK_CSS_VALUE_IMAGE_EFFECT, 1, GTK_CSS_IMAGE_EFFECT_DIM, "dim" }
+  { &GTK_CSS_VALUE_ICON_EFFECT, 1, GTK_CSS_ICON_EFFECT_NONE, "none" },
+  { &GTK_CSS_VALUE_ICON_EFFECT, 1, GTK_CSS_ICON_EFFECT_HIGHLIGHT, "highlight" },
+  { &GTK_CSS_VALUE_ICON_EFFECT, 1, GTK_CSS_ICON_EFFECT_DIM, "dim" }
 };
 
 GtkCssValue *
-_gtk_css_image_effect_value_new (GtkCssImageEffect image_effect)
+_gtk_css_icon_effect_value_new (GtkCssIconEffect image_effect)
 {
   guint i;
 
@@ -900,7 +900,7 @@ _gtk_css_image_effect_value_new (GtkCssImageEffect image_effect)
 }
 
 GtkCssValue *
-_gtk_css_image_effect_value_try_parse (GtkCssParser *parser)
+_gtk_css_icon_effect_value_try_parse (GtkCssParser *parser)
 {
   guint i;
 
@@ -915,14 +915,47 @@ _gtk_css_image_effect_value_try_parse (GtkCssParser *parser)
   return NULL;
 }
 
-GtkCssImageEffect
-_gtk_css_image_effect_value_get (const GtkCssValue *value)
+GtkCssIconEffect
+_gtk_css_icon_effect_value_get (const GtkCssValue *value)
 {
-  g_return_val_if_fail (value->class == &GTK_CSS_VALUE_IMAGE_EFFECT, GTK_CSS_IMAGE_EFFECT_NONE);
+  g_return_val_if_fail (value->class == &GTK_CSS_VALUE_ICON_EFFECT, GTK_CSS_ICON_EFFECT_NONE);
 
   return value->value;
 }
 
+void
+gtk_css_icon_effect_apply (GtkCssIconEffect  icon_effect,
+                           cairo_surface_t  *surface)
+{
+  cairo_t *cr;
+
+  switch (icon_effect)
+    {
+    case GTK_CSS_ICON_EFFECT_DIM:
+      cr = cairo_create (surface);
+      cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+      cairo_set_source_rgba (cr, 0, 0, 0, 0); /* transparent */
+      cairo_paint_with_alpha (cr, 0.5);
+      cairo_destroy (cr);
+      break;
+
+    case GTK_CSS_ICON_EFFECT_HIGHLIGHT:
+      cr = cairo_create (surface);
+      cairo_set_source_rgb (cr, 0.1, 0.1, 0.1);
+      cairo_set_operator (cr, CAIRO_OPERATOR_COLOR_DODGE);
+      /* DANGER: We mask with ourself - that works for images, but... */
+      cairo_mask_surface (cr, surface, 0, 0);
+      cairo_destroy (cr);
+      break;
+
+    default:
+      g_warn_if_reached ();
+      /* fall through */
+    case GTK_CSS_ICON_EFFECT_NONE:
+      break;
+    }
+}
+
 /* GtkCssIconStyle */
 
 static const GtkCssValueClass GTK_CSS_VALUE_ICON_STYLE = {
