@@ -53,6 +53,7 @@
 #include "gtkcssinitialvalueprivate.h"
 #include "gtkcssenumvalueprivate.h"
 #include "gtkcssnumbervalueprivate.h"
+#include "gtkcsspalettevalueprivate.h"
 #include "gtkcsspositionvalueprivate.h"
 #include "gtkcssrepeatvalueprivate.h"
 #include "gtkcssrgbavalueprivate.h"
@@ -497,7 +498,7 @@ static GtkCssValue *
 image_effect_parse (GtkCssStyleProperty *property,
 		    GtkCssParser        *parser)
 {
-  GtkCssValue *value = _gtk_css_image_effect_value_try_parse (parser);
+  GtkCssValue *value = _gtk_css_icon_effect_value_try_parse (parser);
 
   if (value == NULL)
     _gtk_css_parser_error (parser, "unknown value for property");
@@ -505,6 +506,13 @@ image_effect_parse (GtkCssStyleProperty *property,
   return value;
 }
 
+static GtkCssValue *
+icon_palette_parse (GtkCssStyleProperty *property,
+		    GtkCssParser        *parser)
+{
+  return gtk_css_palette_value_parse (parser);
+}
+
 static GtkCssValue *
 icon_style_parse (GtkCssStyleProperty *property,
 		  GtkCssParser        *parser)
@@ -987,9 +995,7 @@ static GtkCssValue *
 icon_theme_value_parse (GtkCssStyleProperty *property,
 		        GtkCssParser        *parser)
 {
-  _gtk_css_parser_error (parser, "Only 'inherit', 'initial' or 'unset' are allowed");
-
-  return NULL;
+  return gtk_css_icon_theme_value_parse (parser);
 }
 
 /*** REGISTRATION ***/
@@ -1028,9 +1034,6 @@ _gtk_css_style_property_init_properties (void)
                                           query_length_as_double,
                                           assign_length_from_double,
                                           _gtk_css_font_size_value_new (GTK_CSS_FONT_SIZE_MEDIUM));
-
-  /* properties that aren't referenced when computing values
-   * start here */
   gtk_css_style_property_register        ("-gtk-icon-theme",
                                           GTK_CSS_PROPERTY_ICON_THEME,
                                           G_TYPE_NONE,
@@ -1039,7 +1042,10 @@ _gtk_css_style_property_init_properties (void)
                                           icon_theme_value_parse,
                                           NULL,
                                           NULL,
-                                          _gtk_css_icon_theme_value_new());
+                                          gtk_css_icon_theme_value_new (NULL));
+
+  /* properties that aren't referenced when computing values
+   * start here */
   gtk_css_style_property_register        ("background-color",
                                           GTK_CSS_PROPERTY_BACKGROUND_COLOR,
                                           GDK_TYPE_RGBA,
@@ -1720,15 +1726,25 @@ _gtk_css_style_property_init_properties (void)
                                           NULL,
                                           NULL,
                                           _gtk_css_number_value_new (1, GTK_CSS_NUMBER));
-  gtk_css_style_property_register        ("-gtk-image-effect",
-					  GTK_CSS_PROPERTY_GTK_IMAGE_EFFECT,
+  gtk_css_style_property_register        ("-gtk-icon-effect",
+					  GTK_CSS_PROPERTY_ICON_EFFECT,
 					  G_TYPE_NONE,
 					  GTK_STYLE_PROPERTY_INHERIT,
                                           GTK_CSS_AFFECTS_ICON,
 					  image_effect_parse,
 					  NULL,
 					  NULL,
-					  _gtk_css_image_effect_value_new (GTK_CSS_IMAGE_EFFECT_NONE));
+					  _gtk_css_icon_effect_value_new (GTK_CSS_ICON_EFFECT_NONE));
+  gtk_css_style_property_register        ("-gtk-icon-palette",
+					  GTK_CSS_PROPERTY_ICON_PALETTE,
+					  G_TYPE_NONE,
+					  GTK_STYLE_PROPERTY_ANIMATED | GTK_STYLE_PROPERTY_INHERIT,
+                                          GTK_CSS_AFFECTS_ICON,
+					  icon_palette_parse,
+					  NULL,
+					  NULL,
+					  gtk_css_palette_value_new_default ());
+
 
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   gtk_css_style_property_register        ("engine",
