@@ -216,7 +216,8 @@ enum { /*< skip >*/
   GTK_CSS_PROPERTY_ANIMATION_DELAY,
   GTK_CSS_PROPERTY_ANIMATION_FILL_MODE,
   GTK_CSS_PROPERTY_OPACITY,
-  GTK_CSS_PROPERTY_GTK_IMAGE_EFFECT,
+  GTK_CSS_PROPERTY_ICON_EFFECT,
+  GTK_CSS_PROPERTY_ICON_PALETTE,
   GTK_CSS_PROPERTY_ENGINE,
   GTK_CSS_PROPERTY_GTK_KEY_BINDINGS,
   /* add more */
@@ -249,10 +250,10 @@ typedef enum /*< skip >*/ {
 } GtkCssFillMode;
 
 typedef enum /*< skip >*/ {
-  GTK_CSS_IMAGE_EFFECT_NONE,
-  GTK_CSS_IMAGE_EFFECT_HIGHLIGHT,
-  GTK_CSS_IMAGE_EFFECT_DIM
-} GtkCssImageEffect;
+  GTK_CSS_ICON_EFFECT_NONE,
+  GTK_CSS_ICON_EFFECT_HIGHLIGHT,
+  GTK_CSS_ICON_EFFECT_DIM
+} GtkCssIconEffect;
 
 typedef enum /*< skip >*/ {
   GTK_CSS_ICON_STYLE_REQUESTED,
