@@ -455,7 +455,10 @@ gtk_image_set_property (GObject      *object,
       break;
     case PROP_ICON_SIZE:
       if (_gtk_icon_helper_set_icon_size (priv->icon_helper, g_value_get_int (value)))
-        g_object_notify_by_pspec (object, pspec);
+        {
+          g_object_notify_by_pspec (object, pspec);
+          gtk_widget_queue_resize (GTK_WIDGET (image));
+        }
       break;
     case PROP_PIXEL_SIZE:
       gtk_image_set_pixel_size (image, g_value_get_int (value));
@@ -1738,6 +1741,63 @@ G_GNUC_END_IGNORE_DEPRECATIONS
   return FALSE;
 }
 
+static void
+gtk_image_notify_for_storage_type (GtkImage     *image,
+                                   GtkImageType  storage_type)
+{
+  switch (storage_type)
+    {
+    case GTK_IMAGE_PIXBUF:
+      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_PIXBUF]);
+      break;
+    case GTK_IMAGE_STOCK:
+      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_STOCK]);
+      break;
+    case GTK_IMAGE_ICON_SET:
+      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_ICON_SET]);
+      break;
+    case GTK_IMAGE_ANIMATION:
+      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_PIXBUF_ANIMATION]);
+      break;
+    case GTK_IMAGE_ICON_NAME:
+      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_ICON_NAME]);
+      break;
+    case GTK_IMAGE_GICON:
+      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_GICON]);
+      break;
+    case GTK_IMAGE_EMPTY:
+    default:
+      break;
+    }
+}
+
+void
+gtk_image_set_from_definition (GtkImage           *image,
+                               GtkImageDefinition *def,
+                               GtkIconSize         icon_size)
+{
+  GtkImagePrivate *priv;
+
+  g_return_if_fail (GTK_IS_IMAGE (image));
+
+  priv = image->priv;
+
+  g_object_freeze_notify (G_OBJECT (image));
+  
+  gtk_image_clear (image);
+
+  if (def != NULL)
+    {
+      _gtk_icon_helper_set_definition (priv->icon_helper, def);
+
+      gtk_image_notify_for_storage_type (image, gtk_image_definition_get_storage_type (def));
+    }
+
+  _gtk_icon_helper_set_icon_size (priv->icon_helper, icon_size);
+  
+  g_object_thaw_notify (G_OBJECT (image));
+}
+
 static void
 gtk_image_reset (GtkImage *image)
 {
@@ -1752,31 +1812,9 @@ gtk_image_reset (GtkImage *image)
 
   g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_ICON_SIZE]);
 
-  switch (storage_type)
-    {
-    case GTK_IMAGE_PIXBUF:
-      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_PIXBUF]);
-      break;
-    case GTK_IMAGE_STOCK:
-      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_STOCK]);
-      break;
-    case GTK_IMAGE_ICON_SET:
-      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_ICON_SET]);
-      break;
-    case GTK_IMAGE_ANIMATION:
-      gtk_image_reset_anim_iter (image);
-      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_PIXBUF_ANIMATION]);
-      break;
-    case GTK_IMAGE_ICON_NAME:
-      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_ICON_NAME]);
-      break;
-    case GTK_IMAGE_GICON:
-      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_GICON]);
-      break;
-    case GTK_IMAGE_EMPTY:
-    default:
-      break;
-    }
+  gtk_image_reset_anim_iter (image);
+
+  gtk_image_notify_for_storage_type (image, storage_type);
 
   if (priv->filename)
     {
@@ -1900,21 +1938,6 @@ gtk_image_screen_changed (GtkWidget *widget,
   icon_theme_changed (image);
 }
 
-void
-_gtk_image_gicon_data_clear (GtkImageGIconData *data)
-{
-  if (data->pixbuf)
-    {
-      g_object_unref (data->pixbuf);
-      data->pixbuf = NULL;
-    }
-  if (data->icon)
-    {
-      g_object_unref (data->icon);
-      data->icon = NULL;
-    }
-}
-
 /**
  * gtk_image_set_pixel_size:
  * @image: a #GtkImage
