@@ -125,7 +125,9 @@ enum {
   PROP_POINTING_TO,
   PROP_POSITION,
   PROP_MODAL,
-  PROP_TRANSITIONS_ENABLED
+  PROP_TRANSITIONS_ENABLED,
+  PROP_CONSTRAIN_TO,
+  NUM_PROPERTIES
 };
 
 enum {
@@ -151,6 +153,7 @@ struct _GtkPopoverPrivate
   GtkAdjustment *vadj;
   GtkAdjustment *hadj;
   GdkRectangle pointing_to;
+  GtkPopoverConstraint constraint;
   guint prev_focus_unmap_id;
   guint hierarchy_changed_id;
   guint size_allocate_id;
@@ -176,6 +179,7 @@ struct _GtkPopoverPrivate
   gint tip_y;
 };
 
+static GParamSpec *properties[NUM_PROPERTIES];
 static GQuark quark_widget_popovers = 0;
 static guint signals[N_SIGNALS] = { 0 };
 
@@ -199,6 +203,8 @@ gtk_popover_init (GtkPopover *popover)
   popover->priv->modal = TRUE;
   popover->priv->tick_id = 0;
   popover->priv->transitions_enabled = TRUE;
+  popover->priv->preferred_position = GTK_POS_TOP;
+  popover->priv->constraint = GTK_POPOVER_CONSTRAINT_WINDOW;
 
   context = gtk_widget_get_style_context (GTK_WIDGET (popover));
   gtk_style_context_add_class (context, GTK_STYLE_CLASS_BACKGROUND);
@@ -232,6 +238,10 @@ gtk_popover_set_property (GObject      *object,
       gtk_popover_set_transitions_enabled (GTK_POPOVER (object),
                                            g_value_get_boolean (value));
       break;
+    case PROP_CONSTRAIN_TO:
+      gtk_popover_set_constrain_to (GTK_POPOVER (object),
+                                    g_value_get_enum (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -262,6 +272,9 @@ gtk_popover_get_property (GObject    *object,
     case PROP_TRANSITIONS_ENABLED:
       g_value_set_boolean (value, priv->transitions_enabled);
       break;
+    case PROP_CONSTRAIN_TO:
+      g_value_set_enum (value, priv->constraint);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -677,6 +690,7 @@ gtk_popover_get_gap_coords (GtkPopover      *popover,
   gint border_radius;
   GtkStyleContext *context;
   GtkBorder margin, border;
+  GtkStateFlags state;
 
   gtk_popover_get_pointing_to (popover, &rect);
   gtk_widget_get_allocation (widget, &allocation);
@@ -708,9 +722,11 @@ gtk_popover_get_gap_coords (GtkPopover      *popover,
   rect.y += gtk_widget_get_margin_top (widget);
 
   context = gtk_widget_get_style_context (widget);
+  state = gtk_style_context_get_state (context);
 
+  gtk_style_context_get_border (context, state, &border);
   gtk_style_context_get (context,
-                         gtk_style_context_get_state (context),
+                         state,
                          GTK_STYLE_PROPERTY_BORDER_RADIUS, &border_radius,
                          NULL);
   pos = get_effective_position (popover, priv->final_position);
@@ -983,7 +999,8 @@ gtk_popover_update_position (GtkPopover *popover)
   overshoot[GTK_POS_RIGHT] = rect.x + rect.width + req.width - window_alloc.width;
 
 #ifdef GDK_WINDOWING_WAYLAND
-  if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (widget)))
+  if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (widget)) &&
+      priv->constraint == GTK_POPOVER_CONSTRAINT_NONE)
     {
       priv->final_position = priv->preferred_position;
     }
@@ -1628,13 +1645,13 @@ gtk_popover_class_init (GtkPopoverClass *klass)
    *
    * Since: 3.12
    */
-  g_object_class_install_property (object_class,
-                                   PROP_RELATIVE_TO,
-                                   g_param_spec_object ("relative-to",
-                                                        P_("Relative to"),
-                                                        P_("Widget the bubble window points to"),
-                                                        GTK_TYPE_WIDGET,
-                                                        GTK_PARAM_READWRITE));
+  properties[PROP_RELATIVE_TO] =
+      g_param_spec_object ("relative-to",
+                           P_("Relative to"),
+                           P_("Widget the bubble window points to"),
+                           GTK_TYPE_WIDGET,
+                           GTK_PARAM_READWRITE);
+
   /**
    * GtkPopover:pointing-to:
    *
@@ -1642,13 +1659,13 @@ gtk_popover_class_init (GtkPopoverClass *klass)
    *
    * Since: 3.12
    */
-  g_object_class_install_property (object_class,
-                                   PROP_POINTING_TO,
-                                   g_param_spec_boxed ("pointing-to",
-                                                       P_("Pointing to"),
-                                                       P_("Rectangle the bubble window points to"),
-                                                       GDK_TYPE_RECTANGLE,
-                                                       GTK_PARAM_READWRITE));
+  properties[PROP_POINTING_TO] =
+      g_param_spec_boxed ("pointing-to",
+                          P_("Pointing to"),
+                          P_("Rectangle the bubble window points to"),
+                          GDK_TYPE_RECTANGLE,
+                          GTK_PARAM_READWRITE);
+
   /**
    * GtkPopover:position
    *
@@ -1656,13 +1673,12 @@ gtk_popover_class_init (GtkPopoverClass *klass)
    *
    * Since: 3.12
    */
-  g_object_class_install_property (object_class,
-                                   PROP_POSITION,
-                                   g_param_spec_enum ("position",
-                                                      P_("Position"),
-                                                      P_("Position to place the bubble window"),
-                                                      GTK_TYPE_POSITION_TYPE, GTK_POS_TOP,
-                                                      GTK_PARAM_READWRITE|G_PARAM_CONSTRUCT|G_PARAM_EXPLICIT_NOTIFY));
+  properties[PROP_POSITION] =
+      g_param_spec_enum ("position",
+                         P_("Position"),
+                         P_("Position to place the bubble window"),
+                         GTK_TYPE_POSITION_TYPE, GTK_POS_TOP,
+                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * GtkPopover:modal
@@ -1672,13 +1688,12 @@ gtk_popover_class_init (GtkPopoverClass *klass)
    *
    * Since: 3.12
    */
-  g_object_class_install_property (object_class,
-                                   PROP_MODAL,
-                                   g_param_spec_boolean ("modal",
-                                                         P_("Modal"),
-                                                         P_("Whether the popover is modal"),
-                                                         TRUE,
-                                                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
+  properties[PROP_MODAL] =
+      g_param_spec_boolean ("modal",
+                            P_("Modal"),
+                            P_("Whether the popover is modal"),
+                            TRUE,
+                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * GtkPopover:transitions-enabled
@@ -1687,13 +1702,28 @@ gtk_popover_class_init (GtkPopoverClass *klass)
    *
    * Since: 3.16
    */
-  g_object_class_install_property (object_class,
-                                   PROP_TRANSITIONS_ENABLED,
-                                   g_param_spec_boolean ("transitions-enabled",
-                                                         P_("Transitions enabled"),
-                                                         P_("Whether show/hide transitions are enabled or not"),
-                                                         TRUE,
-                                                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
+  properties[PROP_TRANSITIONS_ENABLED] =
+      g_param_spec_boolean ("transitions-enabled",
+                            P_("Transitions enabled"),
+                            P_("Whether show/hide transitions are enabled or not"),
+                            TRUE,
+                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * GtkPopover:constrain-to:
+   *
+   * Sets a constraint for the popover position.
+   *
+   * Since: 3.20
+   */
+  properties[PROP_CONSTRAIN_TO] =
+      g_param_spec_enum ("constrain-to",
+                         P_("Constraint"),
+                         P_("Constraint for the popover position"),
+                         GTK_TYPE_POPOVER_CONSTRAINT, GTK_POPOVER_CONSTRAINT_WINDOW,
+                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
+
+  g_object_class_install_properties (object_class, NUM_PROPERTIES, properties);
 
   signals[CLOSED] =
     g_signal_new (I_("closed"),
@@ -1966,7 +1996,7 @@ gtk_popover_update_relative_to (GtkPopover *popover,
     }
 
   priv->widget = relative_to;
-  g_object_notify (G_OBJECT (popover), "relative-to");
+  g_object_notify_by_pspec (G_OBJECT (popover), properties[PROP_RELATIVE_TO]);
 
   if (priv->widget)
     {
@@ -2031,18 +2061,18 @@ gtk_popover_update_pointing_to (GtkPopover         *popover,
   else
     priv->has_pointing_to = FALSE;
 
-  g_object_notify (G_OBJECT (popover), "pointing-to");
+  g_object_notify_by_pspec (G_OBJECT (popover), properties[PROP_POINTING_TO]);
 }
 
 static void
 gtk_popover_update_preferred_position (GtkPopover      *popover,
                                        GtkPositionType  position)
 {
-  if (popover->priv->preferred_position != position)
-    {
-      popover->priv->preferred_position = position;
-      g_object_notify (G_OBJECT (popover), "position");
-    }
+  if (popover->priv->preferred_position == position)
+    return;
+
+  popover->priv->preferred_position = position;
+  g_object_notify_by_pspec (G_OBJECT (popover), properties[PROP_POSITION]);
 }
 
 /**
@@ -2238,7 +2268,7 @@ gtk_popover_set_modal (GtkPopover *popover,
   if (gtk_widget_is_visible (GTK_WIDGET (popover)))
     gtk_popover_apply_modality (popover, priv->modal);
 
-  g_object_notify (G_OBJECT (popover), "modal");
+  g_object_notify_by_pspec (G_OBJECT (popover), properties[PROP_MODAL]);
 }
 
 /**
@@ -2283,7 +2313,7 @@ gtk_popover_set_transitions_enabled (GtkPopover *popover,
     return;
 
   priv->transitions_enabled = transitions_enabled;
-  g_object_notify (G_OBJECT (popover), "transitions-enabled");
+  g_object_notify_by_pspec (G_OBJECT (popover), properties[PROP_TRANSITIONS_ENABLED]);
 }
 
 /**
@@ -2479,3 +2509,53 @@ gtk_popover_get_default_widget (GtkPopover *popover)
 
   return priv->default_widget;
 }
+
+/**
+ * gtk_popover_set_constrain_to:
+ * @popover: a #GtkPopover
+ * @constraint: the new constraint
+ *
+ * Sets a constraint for positioning this popover.
+ *
+ * Note that not all platforms support placing popovers freely,
+ * and may already impose constraints.
+ *
+ * Since: 3.20
+ */
+void
+gtk_popover_set_constrain_to (GtkPopover           *popover,
+                              GtkPopoverConstraint  constraint)
+{
+  GtkPopoverPrivate *priv = popover->priv;
+
+  g_return_if_fail (GTK_IS_POPOVER (popover));
+
+  if (priv->constraint == constraint)
+    return;
+
+  priv->constraint = constraint;
+  gtk_popover_update_position (popover);
+
+  g_object_notify_by_pspec (G_OBJECT (popover), properties[PROP_CONSTRAIN_TO]);
+}
+
+/**
+ * gtk_popover_get_constrain_to:
+ * @popover: a #GtkPopover
+ *
+ * Returns the constraint for placing this popover.
+ * See gtk_popover_set_constrain_to().
+ *
+ * Returns: the constraint for placing this popover.
+ *
+ * Since: 3.20
+ */
+GtkPopoverConstraint
+gtk_popover_get_constrain_to (GtkPopover *popover)
+{
+  GtkPopoverPrivate *priv = popover->priv;
+
+  g_return_val_if_fail (GTK_IS_POPOVER (popover), GTK_POPOVER_CONSTRAINT_WINDOW);
+
+  return priv->constraint;
+}
