@@ -31,6 +31,10 @@
 #include "gtkwidgetprivate.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
+#include "gtkcssshadowsvalueprivate.h"
+#include "gtkstylecontextprivate.h"
+#include "gtkcssnodeprivate.h"
+#include "gtkcssstylepropertyprivate.h"
 
 #include "a11y/gtkprogressbaraccessible.h"
 
@@ -61,6 +65,20 @@
  * of the #GtkProgressBar. Functions are provided to control the orientation
  * of the bar, optional text can be displayed along with the bar, and the
  * step size used in activity mode can be set.
+ *
+ * # CSS nodes
+ *
+ * |[<!-- language="plain" -->
+ * progressbar
+ * ╰── trough
+ *     ╰── progress[.pulse]
+ * ]|
+ *
+ * GtkProgressBar has a main CSS node with name progressbar and subnodes
+ * with names trough and progress. The progress subnode has the style class
+ * .pulse when in activity mode. It gets the style classes .left, .right,
+ * .top or .bottom added when the progress 'touches' the corresponding end
+ * of the GtkProgressBar.
  */
 
 #define MIN_HORIZONTAL_BAR_WIDTH   150
@@ -73,6 +91,9 @@ struct _GtkProgressBarPrivate
 {
   gchar         *text;
 
+  GtkCssNode    *trough_node;
+  GtkCssNode    *progress_node;
+
   gdouble        fraction;
   gdouble        pulse_fraction;
 
@@ -97,13 +118,16 @@ enum {
   PROP_0,
   PROP_FRACTION,
   PROP_PULSE_STEP,
-  PROP_ORIENTATION,
   PROP_INVERTED,
   PROP_TEXT,
   PROP_SHOW_TEXT,
-  PROP_ELLIPSIZE
+  PROP_ELLIPSIZE,
+  PROP_ORIENTATION,
+  NUM_PROPERTIES = PROP_ORIENTATION
 };
 
+static GParamSpec *progress_props[NUM_PROPERTIES] = { NULL, };
+
 static void gtk_progress_bar_set_property         (GObject        *object,
                                                    guint           prop_id,
                                                    const GValue   *value,
@@ -128,6 +152,8 @@ static void     gtk_progress_bar_act_mode_leave   (GtkProgressBar *progress);
 static void     gtk_progress_bar_finalize         (GObject        *object);
 static void     gtk_progress_bar_set_orientation  (GtkProgressBar *progress,
                                                    GtkOrientation  orientation);
+static void     gtk_progress_bar_direction_changed (GtkWidget        *widget,
+                                                    GtkTextDirection  previous_dir);
 
 G_DEFINE_TYPE_WITH_CODE (GtkProgressBar, gtk_progress_bar, GTK_TYPE_WIDGET,
                          G_ADD_PRIVATE (GtkProgressBar)
@@ -150,40 +176,39 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   widget_class->size_allocate = gtk_progress_bar_size_allocate;
   widget_class->get_preferred_width = gtk_progress_bar_get_preferred_width;
   widget_class->get_preferred_height = gtk_progress_bar_get_preferred_height;
+  widget_class->direction_changed = gtk_progress_bar_direction_changed;
 
   g_object_class_override_property (gobject_class, PROP_ORIENTATION, "orientation");
 
-  g_object_class_install_property (gobject_class,
-                                   PROP_INVERTED,
-                                   g_param_spec_boolean ("inverted",
-                                                         P_("Inverted"),
-                                                         P_("Invert the direction in which the progress bar grows"),
-                                                         FALSE,
-                                                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
+  progress_props[PROP_INVERTED] =
+      g_param_spec_boolean ("inverted",
+                            P_("Inverted"),
+                            P_("Invert the direction in which the progress bar grows"),
+                            FALSE,
+                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
-  g_object_class_install_property (gobject_class,
-                                   PROP_FRACTION,
-                                   g_param_spec_double ("fraction",
-                                                        P_("Fraction"),
-                                                        P_("The fraction of total work that has been completed"),
-                                                        0.0, 1.0, 0.0,
-                                                        GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
+  progress_props[PROP_FRACTION] =
+      g_param_spec_double ("fraction",
+                           P_("Fraction"),
+                           P_("The fraction of total work that has been completed"),
+                           0.0, 1.0,
+                           0.0,
+                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
-  g_object_class_install_property (gobject_class,
-                                   PROP_PULSE_STEP,
-                                   g_param_spec_double ("pulse-step",
-                                                        P_("Pulse Step"),
-                                                        P_("The fraction of total progress to move the bouncing block when pulsed"),
-                                                        0.0, 1.0, 0.1,
-                                                        GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
+  progress_props[PROP_PULSE_STEP] =
+      g_param_spec_double ("pulse-step",
+                           P_("Pulse Step"),
+                           P_("The fraction of total progress to move the bouncing block when pulsed"),
+                           0.0, 1.0,
+                           0.1,
+                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
-  g_object_class_install_property (gobject_class,
-                                   PROP_TEXT,
-                                   g_param_spec_string ("text",
-                                                        P_("Text"),
-                                                        P_("Text to be displayed in the progress bar"),
-                                                        NULL,
-                                                        GTK_PARAM_READWRITE));
+  progress_props[PROP_TEXT] =
+      g_param_spec_string ("text",
+                           P_("Text"),
+                           P_("Text to be displayed in the progress bar"),
+                           NULL,
+                           GTK_PARAM_READWRITE);
 
   /**
    * GtkProgressBar:show-text:
@@ -200,13 +225,12 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
    *
    * Since: 3.0
    */
-  g_object_class_install_property (gobject_class,
-                                   PROP_SHOW_TEXT,
-                                   g_param_spec_boolean ("show-text",
-                                                         P_("Show text"),
-                                                         P_("Whether the progress is shown as text."),
-                                                         FALSE,
-                                                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
+  progress_props[PROP_SHOW_TEXT] =
+      g_param_spec_boolean ("show-text",
+                            P_("Show text"),
+                            P_("Whether the progress is shown as text."),
+                            FALSE,
+                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * GtkProgressBar:ellipsize:
@@ -222,29 +246,46 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
    *
    * Since: 2.6
    */
-  g_object_class_install_property (gobject_class,
-                                   PROP_ELLIPSIZE,
-                                   g_param_spec_enum ("ellipsize",
-                                                      P_("Ellipsize"),
-                                                      P_("The preferred place to ellipsize the string, if the progress bar "
-                                                         "does not have enough room to display the entire string, if at all."),
-                                                      PANGO_TYPE_ELLIPSIZE_MODE,
-                                                      PANGO_ELLIPSIZE_NONE,
-                                                      GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
+  progress_props[PROP_ELLIPSIZE] =
+      g_param_spec_enum ("ellipsize",
+                         P_("Ellipsize"),
+                         P_("The preferred place to ellipsize the string, if the progress bar "
+                            "does not have enough room to display the entire string, if at all."),
+                         PANGO_TYPE_ELLIPSIZE_MODE,
+                         PANGO_ELLIPSIZE_NONE,
+                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
+  g_object_class_install_properties (gobject_class, NUM_PROPERTIES, progress_props);
+
+  /**
+   * GtkProgressBar:xspacing:
+   *
+   * Extra spacing applied to the width of a progress bar.
+   *
+   * Deprecated: 3.20: Use the standard CSS padding and margins; the
+   *     value of this style property is ignored.
+   */
   gtk_widget_class_install_style_property (widget_class,
                                            g_param_spec_int ("xspacing",
                                                              P_("X spacing"),
                                                              P_("Extra spacing applied to the width of a progress bar."),
                                                              0, G_MAXINT, 2,
-                                                             G_PARAM_READWRITE));
+                                                             G_PARAM_READWRITE|G_PARAM_DEPRECATED));
 
+  /**
+   * GtkProgressBar:yspacing:
+   *
+   * Extra spacing applied to the height of a progress bar.
+   *
+   * Deprecated: 3.20: Use the standard CSS padding and margins; the
+   *     value of this style property is ignored.
+   */
   gtk_widget_class_install_style_property (widget_class,
                                            g_param_spec_int ("yspacing",
                                                              P_("Y spacing"),
                                                              P_("Extra spacing applied to the height of a progress bar."),
                                                              0, G_MAXINT, 2,
-                                                             G_PARAM_READWRITE));
+                                                             G_PARAM_READWRITE|G_PARAM_DEPRECATED));
 
   /**
    * GtkProgressBar:min-horizontal-bar-width:
@@ -300,13 +341,107 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
                                                              G_PARAM_READWRITE));
 
   gtk_widget_class_set_accessible_type (widget_class, GTK_TYPE_PROGRESS_BAR_ACCESSIBLE);
+  gtk_widget_class_set_css_name (widget_class, "progressbar");
+}
+
+static void
+node_style_changed_cb (GtkCssNode  *node,
+                       GtkCssStyle *old_style,
+                       GtkCssStyle *new_style,
+                       GtkWidget   *widget)
+{
+  GtkBitmask *changes;
+  static GtkBitmask *affects_size = NULL;
+
+  if (G_UNLIKELY (affects_size == NULL))
+    affects_size = _gtk_css_style_property_get_mask_affecting (GTK_CSS_AFFECTS_SIZE | GTK_CSS_AFFECTS_CLIP);
+
+  changes = _gtk_bitmask_new ();
+  changes = gtk_css_style_add_difference (changes, old_style, new_style);
+
+  if (_gtk_bitmask_intersects (changes, affects_size))
+    gtk_widget_queue_resize (widget);
+  else
+    gtk_widget_queue_draw (widget);
+
+  _gtk_bitmask_free (changes);
+}
+
+static void
+update_node_classes (GtkProgressBar *pbar)
+{
+  GtkProgressBarPrivate *priv = pbar->priv;
+  gboolean left = FALSE;
+  gboolean right = FALSE;
+  gboolean top = FALSE;
+  gboolean bottom = FALSE;
+
+  /* Here we set positional classes, depending on which end of the
+   * progressbar the progress touches.
+   */
+
+  if (priv->activity_mode)
+    {
+      if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+        {
+          left = priv->activity_pos <= 0.0;
+          right = priv->activity_pos >= 1.0;
+        }
+      else
+        {
+          top = priv->activity_pos <= 0.0;
+          bottom = priv->activity_pos >= 1.0;
+        }
+    }
+  else /* continuous */
+    {
+      gboolean inverted;
+
+      inverted = priv->inverted;
+      if (gtk_widget_get_direction (GTK_WIDGET (pbar)) == GTK_TEXT_DIR_RTL)
+        {
+          if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+            inverted = !inverted;
+        }
+
+      if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+        {
+          left = !inverted || priv->fraction >= 1.0;
+          right = inverted || priv->fraction >= 1.0;
+        }
+      else
+        {
+          top = !inverted || priv->fraction >= 1.0;
+          bottom = inverted || priv->fraction >= 1.0;
+        }
+    }
+
+  if (left)
+    gtk_css_node_add_class (priv->progress_node, g_quark_from_static_string (GTK_STYLE_CLASS_LEFT));
+  else
+    gtk_css_node_remove_class (priv->progress_node, g_quark_from_static_string (GTK_STYLE_CLASS_LEFT));
+
+  if (right)
+    gtk_css_node_add_class (priv->progress_node, g_quark_from_static_string (GTK_STYLE_CLASS_RIGHT));
+  else
+    gtk_css_node_remove_class (priv->progress_node, g_quark_from_static_string (GTK_STYLE_CLASS_RIGHT));
+
+  if (top)
+    gtk_css_node_add_class (priv->progress_node, g_quark_from_static_string (GTK_STYLE_CLASS_TOP));
+  else
+    gtk_css_node_remove_class (priv->progress_node, g_quark_from_static_string (GTK_STYLE_CLASS_TOP));
+
+  if (bottom)
+    gtk_css_node_add_class (priv->progress_node, g_quark_from_static_string (GTK_STYLE_CLASS_BOTTOM));
+  else
+    gtk_css_node_remove_class (priv->progress_node, g_quark_from_static_string (GTK_STYLE_CLASS_BOTTOM));
 }
 
 static void
 gtk_progress_bar_init (GtkProgressBar *pbar)
 {
   GtkProgressBarPrivate *priv;
-  GtkStyleContext *context;
+  GtkCssNode *widget_node;
 
   pbar->priv = gtk_progress_bar_get_instance_private (pbar);
   priv = pbar->priv;
@@ -324,9 +459,26 @@ gtk_progress_bar_init (GtkProgressBar *pbar)
   priv->fraction = 0.0;
 
   gtk_widget_set_has_window (GTK_WIDGET (pbar), FALSE);
+
   _gtk_orientable_set_style_classes (GTK_ORIENTABLE (pbar));
-  context = gtk_widget_get_style_context (GTK_WIDGET (pbar));
-  gtk_style_context_add_class (context, GTK_STYLE_CLASS_TROUGH);
+
+  widget_node = gtk_widget_get_css_node (GTK_WIDGET (pbar));
+
+  priv->trough_node = gtk_css_node_new ();
+  gtk_css_node_set_name (priv->trough_node, I_("trough"));
+  gtk_css_node_set_parent (priv->trough_node, widget_node);
+  gtk_css_node_set_state (priv->trough_node, gtk_css_node_get_state (widget_node));
+  g_signal_connect_object (priv->trough_node, "style-changed", G_CALLBACK (node_style_changed_cb), pbar, 0);
+  g_object_unref (priv->trough_node);
+
+  priv->progress_node = gtk_css_node_new ();
+  gtk_css_node_set_name (priv->progress_node, I_("progress"));
+  gtk_css_node_set_parent (priv->progress_node, priv->trough_node);
+  gtk_css_node_set_state (priv->progress_node, gtk_css_node_get_state (widget_node));
+  g_signal_connect_object (priv->progress_node, "style-changed", G_CALLBACK (node_style_changed_cb), pbar, 0);
+  g_object_unref (priv->progress_node);
+
+  update_node_classes (pbar);
 }
 
 static void
@@ -464,21 +616,18 @@ gtk_progress_bar_get_preferred_width (GtkWidget *widget,
 {
   GtkProgressBar *pbar;
   GtkProgressBarPrivate *priv;
-  GtkStyleContext *style_context;
-  GtkStateFlags state;
+  GtkStyleContext *context;
   GtkBorder padding;
   gchar *buf;
   PangoRectangle logical_rect;
   PangoLayout *layout;
   gint width;
-  gint xspacing;
   gint bar_width;
 
   g_return_if_fail (GTK_IS_PROGRESS_BAR (widget));
 
-  style_context = gtk_widget_get_style_context (widget);
-  state = gtk_widget_get_state_flags (widget);
-  gtk_style_context_get_padding (style_context, state, &padding);
+  context = gtk_widget_get_style_context (widget);
+  gtk_style_context_get_padding (context, gtk_style_context_get_state (context), &padding);
 
   pbar = GTK_PROGRESS_BAR (widget);
   priv = pbar->priv;
@@ -487,11 +636,6 @@ gtk_progress_bar_get_preferred_width (GtkWidget *widget,
 
   if (priv->show_text)
     {
-      gtk_widget_style_get (widget,
-                            "xspacing", &xspacing,
-                            NULL);
-      width += xspacing;
-
       buf = get_current_text (pbar);
       layout = gtk_widget_create_pango_layout (widget, buf);
 
@@ -541,20 +685,17 @@ gtk_progress_bar_get_preferred_height (GtkWidget *widget,
   GtkProgressBar *pbar;
   GtkProgressBarPrivate *priv;
   GtkStyleContext *context;
-  GtkStateFlags state;
   GtkBorder padding;
   gchar *buf;
   PangoRectangle logical_rect;
   PangoLayout *layout;
   gint height;
-  gint yspacing;
   gint bar_height;
 
   g_return_if_fail (GTK_IS_PROGRESS_BAR (widget));
 
   context = gtk_widget_get_style_context (widget);
-  state = gtk_widget_get_state_flags (widget);
-  gtk_style_context_get_padding (context, state, &padding);
+  gtk_style_context_get_padding (context, gtk_style_context_get_state (context), &padding);
 
   pbar = GTK_PROGRESS_BAR (widget);
   priv = pbar->priv;
@@ -563,11 +704,6 @@ gtk_progress_bar_get_preferred_height (GtkWidget *widget,
 
   if (priv->show_text)
     {
-      gtk_widget_style_get (widget,
-                            "yspacing", &yspacing,
-                            NULL);
-      height += yspacing;
-
       buf = get_current_text (pbar);
       layout = gtk_widget_create_pango_layout (widget, buf);
 
@@ -644,6 +780,8 @@ tick_cb (GtkWidget     *widget,
         }
     }
 
+  update_node_classes (pbar);
+
   gtk_widget_queue_draw (widget);
 
   return G_SOURCE_CONTINUE;
@@ -653,16 +791,11 @@ static void
 gtk_progress_bar_act_mode_enter (GtkProgressBar *pbar)
 {
   GtkProgressBarPrivate *priv = pbar->priv;
-  GtkStyleContext *context;
-  GtkStateFlags state;
-  GtkBorder padding;
   GtkWidget *widget = GTK_WIDGET (pbar);
   GtkOrientation orientation;
   gboolean inverted;
 
-  context = gtk_widget_get_style_context (widget);
-  state = gtk_widget_get_state_flags (widget);
-  gtk_style_context_get_padding (context, state, &padding);
+  gtk_css_node_add_class (priv->progress_node, g_quark_from_static_string (GTK_STYLE_CLASS_PULSE));
 
   orientation = priv->orientation;
   inverted = priv->inverted;
@@ -701,6 +834,7 @@ gtk_progress_bar_act_mode_enter (GtkProgressBar *pbar)
         }
     }
 
+  update_node_classes (pbar);
   priv->tick_id = gtk_widget_add_tick_callback (widget, tick_cb, NULL, NULL);
   priv->pulse2 = 0;
   priv->pulse1 = 0;
@@ -715,6 +849,9 @@ gtk_progress_bar_act_mode_leave (GtkProgressBar *pbar)
   if (priv->tick_id)
     gtk_widget_remove_tick_callback (GTK_WIDGET (pbar), priv->tick_id);
   priv->tick_id = 0;
+
+  gtk_css_node_remove_class (priv->progress_node, g_quark_from_static_string (GTK_STYLE_CLASS_PULSE));
+  update_node_classes (pbar);
 }
 
 static void
@@ -755,40 +892,28 @@ gtk_progress_bar_paint_activity (GtkProgressBar *pbar,
                                  int             width,
                                  int             height)
 {
+  GtkProgressBarPrivate *priv = pbar->priv;
   GtkStyleContext *context;
-  GtkStateFlags state;
   GtkBorder padding;
   GtkWidget *widget = GTK_WIDGET (pbar);
   GdkRectangle area;
 
   context = gtk_widget_get_style_context (widget);
-  state = gtk_widget_get_state_flags (widget);
-  gtk_style_context_get_padding (context, state, &padding);
+  gtk_style_context_get_padding (context, gtk_style_context_get_state (context), &padding);
 
-  gtk_style_context_save (context);
-  gtk_style_context_remove_class (context, GTK_STYLE_CLASS_TROUGH);
-  gtk_style_context_add_class (context, GTK_STYLE_CLASS_PROGRESSBAR);
-  gtk_style_context_add_class (context, GTK_STYLE_CLASS_PULSE);
+  gtk_style_context_save_to_node (context, priv->progress_node);
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
     {
       gtk_progress_bar_get_activity (pbar, orientation, &area.x, &area.width);
       area.y = y + padding.top;
       area.height = height - padding.top - padding.bottom;
-      if (pbar->priv->activity_pos <= 0.0)
-        gtk_style_context_add_class (context, GTK_STYLE_CLASS_LEFT);
-      else if (pbar->priv->activity_pos >= 1.0)
-        gtk_style_context_add_class (context, GTK_STYLE_CLASS_RIGHT);
     }
   else
     {
       gtk_progress_bar_get_activity (pbar, orientation, &area.y, &area.height);
       area.x = x + padding.left;
       area.width = width - padding.left - padding.right;
-      if (pbar->priv->activity_pos <= 0.0)
-        gtk_style_context_add_class (context, GTK_STYLE_CLASS_TOP);
-      else if (pbar->priv->activity_pos >= 1.0)
-        gtk_style_context_add_class (context, GTK_STYLE_CLASS_BOTTOM);
     }
 
   gtk_render_background (context, cr, area.x, area.y, area.width, area.height);
@@ -808,8 +933,8 @@ gtk_progress_bar_paint_continuous (GtkProgressBar *pbar,
                                    int             width,
                                    int             height)
 {
+  GtkProgressBarPrivate *priv = pbar->priv;
   GtkStyleContext *context;
-  GtkStateFlags state;
   GtkBorder padding;
   GtkWidget *widget = GTK_WIDGET (pbar);
   GdkRectangle area;
@@ -818,12 +943,9 @@ gtk_progress_bar_paint_continuous (GtkProgressBar *pbar,
     return;
 
   context = gtk_widget_get_style_context (widget);
-  state = gtk_widget_get_state_flags (widget);
-  gtk_style_context_get_padding (context, state, &padding);
+  gtk_style_context_get_padding (context, gtk_style_context_get_state (context), &padding);
 
-  gtk_style_context_save (context);
-  gtk_style_context_remove_class (context, GTK_STYLE_CLASS_TROUGH);
-  gtk_style_context_add_class (context, GTK_STYLE_CLASS_PROGRESSBAR);
+  gtk_style_context_save_to_node (context, priv->progress_node);
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
     {
@@ -835,11 +957,6 @@ gtk_progress_bar_paint_continuous (GtkProgressBar *pbar,
         area.x = x + padding.left;
       else
         area.x = width - amount - padding.right;
-
-      if (!inverted || gtk_progress_bar_get_fraction (pbar) == 1.0)
-        gtk_style_context_add_class (context, GTK_STYLE_CLASS_LEFT);
-      if (inverted || gtk_progress_bar_get_fraction (pbar) == 1.0)
-        gtk_style_context_add_class (context, GTK_STYLE_CLASS_RIGHT);
     }
   else
     {
@@ -851,11 +968,6 @@ gtk_progress_bar_paint_continuous (GtkProgressBar *pbar,
         area.y = y + padding.top;
       else
         area.y = height - amount - padding.bottom;
-
-      if (!inverted || gtk_progress_bar_get_fraction (pbar) == 1.0)
-        gtk_style_context_add_class (context, GTK_STYLE_CLASS_TOP);
-      if (inverted || gtk_progress_bar_get_fraction (pbar) == 1.0)
-        gtk_style_context_add_class (context, GTK_STYLE_CLASS_BOTTOM);
     }
 
   gtk_render_background (context, cr, area.x, area.y, area.width, area.height);
@@ -876,7 +988,6 @@ gtk_progress_bar_paint_text (GtkProgressBar *pbar,
 {
   GtkProgressBarPrivate *priv = pbar->priv;
   GtkStyleContext *context;
-  GtkStateFlags state;
   GtkBorder padding;
   GtkWidget *widget = GTK_WIDGET (pbar);
   gint x;
@@ -888,8 +999,7 @@ gtk_progress_bar_paint_text (GtkProgressBar *pbar,
   GdkRectangle prelight_clip, start_clip, end_clip;
 
   context = gtk_widget_get_style_context (widget);
-  state = gtk_widget_get_state_flags (widget);
-  gtk_style_context_get_padding (context, state, &padding);
+  gtk_style_context_get_padding (context, gtk_style_context_get_state (context), &padding);
 
   buf = get_current_text (pbar);
 
@@ -989,13 +1099,8 @@ gtk_progress_bar_paint_text (GtkProgressBar *pbar,
   gdk_cairo_rectangle (cr, &prelight_clip);
   cairo_clip (cr);
 
-  gtk_style_context_save (context);
-  gtk_style_context_remove_class (context, GTK_STYLE_CLASS_TROUGH);
-  gtk_style_context_add_class (context, GTK_STYLE_CLASS_PROGRESSBAR);
-
   gtk_render_layout (context, cr, x, y, layout);
 
-  gtk_style_context_restore (context);
   cairo_restore (cr);
 
   g_object_unref (layout);
@@ -1011,14 +1116,12 @@ gtk_progress_bar_draw (GtkWidget *widget,
   GtkOrientation orientation;
   gboolean inverted;
   GtkStyleContext *context;
-  GtkStateFlags state;
   GtkBorder padding;
   int width, height;
   int bar_width, bar_height;
 
   context = gtk_widget_get_style_context (widget);
-  state = gtk_widget_get_state_flags (widget);
-  gtk_style_context_get_padding (context, state, &padding);
+  gtk_style_context_get_padding (context, gtk_style_context_get_state (context), &padding);
 
   orientation = priv->orientation;
   inverted = priv->inverted;
@@ -1041,9 +1144,13 @@ gtk_progress_bar_draw (GtkWidget *widget,
       bar_height = height;
     }
 
+  gtk_style_context_save_to_node (context, priv->trough_node);
+
   gtk_render_background (context, cr, width - bar_width, height - bar_height, bar_width, bar_height);
   gtk_render_frame (context, cr, width - bar_width, height - bar_height, bar_width, bar_height);
 
+  gtk_style_context_restore (context);
+
   if (priv->activity_mode)
     {
       gtk_progress_bar_paint_activity (pbar, cr,
@@ -1130,7 +1237,7 @@ gtk_progress_bar_set_fraction (GtkProgressBar *pbar,
   gtk_progress_bar_set_activity_mode (pbar, FALSE);
   gtk_widget_queue_draw (GTK_WIDGET (pbar));
 
-  g_object_notify (G_OBJECT (pbar), "fraction");
+  g_object_notify_by_pspec (G_OBJECT (pbar), progress_props[PROP_FRACTION]);
 }
 
 static void
@@ -1196,7 +1303,7 @@ gtk_progress_bar_set_text (GtkProgressBar *pbar,
 
   gtk_widget_queue_resize (GTK_WIDGET (pbar));
 
-  g_object_notify (G_OBJECT (pbar), "text");
+  g_object_notify_by_pspec (G_OBJECT (pbar), progress_props[PROP_TEXT]);
 }
 
 /**
@@ -1233,7 +1340,7 @@ gtk_progress_bar_set_show_text (GtkProgressBar *pbar,
 
       gtk_widget_queue_resize (GTK_WIDGET (pbar));
 
-      g_object_notify (G_OBJECT (pbar), "show-text");
+      g_object_notify_by_pspec (G_OBJECT (pbar), progress_props[PROP_SHOW_TEXT]);
     }
 }
 
@@ -1276,7 +1383,18 @@ gtk_progress_bar_set_pulse_step (GtkProgressBar *pbar,
 
   priv->pulse_fraction = fraction;
 
-  g_object_notify (G_OBJECT (pbar), "pulse-step");
+  g_object_notify_by_pspec (G_OBJECT (pbar), progress_props[PROP_PULSE_STEP]);
+}
+
+static void
+gtk_progress_bar_direction_changed (GtkWidget        *widget,
+                                    GtkTextDirection  previous_dir)
+{
+  GtkProgressBar *pbar = GTK_PROGRESS_BAR (widget);
+
+  update_node_classes (pbar);
+
+  GTK_WIDGET_CLASS (gtk_progress_bar_parent_class)->direction_changed (widget, previous_dir);
 }
 
 static void
@@ -1289,6 +1407,7 @@ gtk_progress_bar_set_orientation (GtkProgressBar *pbar,
     {
       priv->orientation = orientation;
       _gtk_orientable_set_style_classes (GTK_ORIENTABLE (pbar));
+      update_node_classes (pbar);
       gtk_widget_queue_resize (GTK_WIDGET (pbar));
       g_object_notify (G_OBJECT (pbar), "orientation");
     }
@@ -1316,9 +1435,11 @@ gtk_progress_bar_set_inverted (GtkProgressBar *pbar,
     {
       priv->inverted = inverted;
 
+      update_node_classes (pbar);
+
       gtk_widget_queue_resize (GTK_WIDGET (pbar));
 
-      g_object_notify (G_OBJECT (pbar), "inverted");
+      g_object_notify_by_pspec (G_OBJECT (pbar), progress_props[PROP_INVERTED]);
     }
 }
 
@@ -1416,7 +1537,7 @@ gtk_progress_bar_set_ellipsize (GtkProgressBar     *pbar,
     {
       priv->ellipsize = mode;
 
-      g_object_notify (G_OBJECT (pbar), "ellipsize");
+      g_object_notify_by_pspec (G_OBJECT (pbar), progress_props[PROP_ELLIPSIZE]);
       gtk_widget_queue_resize (GTK_WIDGET (pbar));
     }
 }
