@@ -36,6 +36,11 @@ const char *    gtk_style_context_get_id                     (GtkStyleContext *c
 GtkStyleProviderPrivate *
                 gtk_style_context_get_style_provider         (GtkStyleContext *context);
 
+void            gtk_style_context_save_named                 (GtkStyleContext *context,
+                                                              const char      *name);
+void            gtk_style_context_save_to_node               (GtkStyleContext *context,
+                                                              GtkCssNode      *node);
+
 const GtkBitmask *
                 _gtk_style_context_get_changes               (GtkStyleContext *context);
 
@@ -64,6 +69,8 @@ void           _gtk_style_context_get_icon_extents           (GtkStyleContext
                                                               gint                width,
                                                               gint                height);
 
+gboolean       _gtk_style_context_is_background_opaque       (GtkStyleContext *context);
+
 PangoAttrList *_gtk_style_context_get_pango_attributes       (GtkStyleContext *context);
 
 /* Accessibility support */
