@@ -182,6 +182,8 @@ GDK_AVAILABLE_IN_ALL GType gtk_event_sequence_state_get_type (void) G_GNUC_CONST
 #define GTK_TYPE_EVENT_SEQUENCE_STATE (gtk_event_sequence_state_get_type ())
 GDK_AVAILABLE_IN_ALL GType gtk_pan_direction_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PAN_DIRECTION (gtk_pan_direction_get_type ())
+GDK_AVAILABLE_IN_ALL GType gtk_popover_constraint_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_POPOVER_CONSTRAINT (gtk_popover_constraint_get_type ())
 
 /* enumerations from "gtkfilechooser.h" */
 GDK_AVAILABLE_IN_ALL GType gtk_file_chooser_action_get_type (void) G_GNUC_CONST;
